package net.locusworks.discord.eighttrack.listeners;

import java.util.Date;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import net.dv8tion.jda.api.events.guild.GuildJoinEvent;
import net.locusworks.discord.eighttrack.database.entities.DiscordGuild;
import net.locusworks.discord.eighttrack.services.RepositoryService;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

@Service
public class DiscordGuildJoinEventListener extends AbstractDiscordEventListener {
  
  private ApplicationLogger logger = ApplicationLoggerFactory.getLogger(DiscordGuildJoinEventListener.class);
  
  @Autowired
  private RepositoryService guildSongRepoService;
  
  @Override
  public void onGuildJoin(GuildJoinEvent event) {
    try {
      long guildId = event.getGuild().getIdLong();
      DiscordGuild discordGuild =  guildSongRepoService.getGuildRepo().findByGuildId(guildId);
      if (discordGuild != null) {
        guildSongRepoService.logEntry(guildId, "Eight Track joined guild %s", discordGuild.getGuildName());
        return;
      }

      logger.debug("Joining Server: " + event.getGuild().getName());

      discordGuild = new DiscordGuild();
      discordGuild.setGuildId(guildId);
      discordGuild.setGuildName(event.getGuild().getName());
      discordGuild.setDateJoined(new Date());

      guildSongRepoService.getGuildRepo().save(discordGuild);
      
      guildSongRepoService.logEntry(guildId, "Eight Track joined guild %s", discordGuild.getGuildName());
      
    } catch (Exception ex) {
      logger.error("Unable to persist server information to database: " + ex.getMessage(), ex);
      guildSongRepoService.logEntry(event.getGuild().getIdLong(), ex, "Unable to persist guild server info to database: %s", ex.getMessage());
    }
  }

}
