package net.locusworks.discord.eighttrack.utils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanInitializationException;
import org.springframework.beans.factory.annotation.Autowire;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.ClassPathScanningCandidateComponentProvider;
import org.springframework.core.type.classreading.MetadataReader;
import org.springframework.core.type.classreading.MetadataReaderFactory;
import org.springframework.core.type.filter.TypeFilter;

public class BeanFinder {
  
  
  public static List<BeanDefinition> getBeans(String basePackage, TypeFilter... filters) {
    ClassPathScanningCandidateComponentProvider scanner = new ClassPathScanningCandidateComponentProvider(false);
    scanner.addIncludeFilter(new TypeFilter() {
      
      @Override
      public boolean match(MetadataReader metadataReader, MetadataReaderFactory metadataReaderFactory) throws IOException {
        boolean found = true;
        
        for (TypeFilter filter : filters) {
          found &= filter.match(metadataReader, metadataReaderFactory);
        }
        
        return found;
      }
    });
    
    return scanner.findCandidateComponents(basePackage).stream().collect(Collectors.toList());
  }
  
  @SuppressWarnings("unchecked")
  public static <T> List<T> autowireBeans(Class<T> clazz, String basePackage, TypeFilter... filters) throws BeansException {
    try {
      List<T> list = new ArrayList<>();
      for (BeanDefinition bd : getBeans(basePackage, filters)) {
        Object obj = ApplicationContextHolder.getContext().getAutowireCapableBeanFactory().createBean(Class.forName(bd.getBeanClassName()), Autowire.BY_TYPE.value(), true);
        if (clazz.isInstance(obj)) {
          list.add((T) obj);
        }
      }
      return list;
    } catch (Exception ex) {
      throw new BeanInitializationException(ex.getMessage(), ex);
    }
  }

}
